<?php
namespace Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\Controller;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DTO\Response\Notification;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Zeedhi\Framework\DB\StoredProcedure\Param;

class Gop11000AprovaOrcaOperacController extends \Teknisa\GER\Controllers\TeknisaCrud{
    protected $dataSourceName = "gop11_teste";
    protected $dataSourceManager;
    protected $connection;
    protected $newCode;
    protected $environment;

    
    public function __construct($crudService, ParameterBag $parameterBag, $dataSourceManager, $connection, $service, $newCode, $environment) {
        $this->dataSourceManager    = $dataSourceManager;
        $this->connection = $connection;
        $this->service = $service;
        $this->newCode = $newCode;
        $this->environment = $environment;
        parent::__construct($crudService, $parameterBag);
    }
    
    public function saveAprova(DTO\Request $request, DTO\Response $response ){
       $data = $request->getRow();
       $rows = $data['ROWS'];
       $flag = $data['FLAG'];
       $observacao = $data['OBSERVACAO'];
        
       try{
           $this->service->beginTransaction();
           foreach($rows as $row) {
               if($flag == 'A') {
                   if(isset($row['NRAPROVAORCAGOP'])) {
                      $this->updateAprovacao($row, 'A', $observacao);
                   }else {
                       $params = array(
                           'P_NRORCAMENTOGOP'       => $row['NRORCAMENTOGOP'],
                           'P_NRAPROVAORCAGOP'      => $this->newCode->getCodeByTableName('APROVAORCAGOP', $this->environment->getNrorg()),
                           'P_CDOPERADOR'           => $this->environment->getCdOperador(),
                           'P_IDAPROVACAOGOP'       => 'APROVADO',
                           'P_DSOBSERVACAO'         => $observacao,
                           'P_NRORG'                => $this->environment->getNrorg(),
                           'P_DTINCLUSAO'           => $this->crudService->getEnvironment()->getCurrentTime()->format('d/m/Y H:i:s'),
                           'P_NRORGINCLUSAO'        => $this->environment->getNrorg(),
                           'P_CDOPERINCLUSAO'       => $this->environment->getCdOperador(),
                           'P_IDTIPOORCAOP'         => $row['IDTIPOORCAOP']
                        );
                        $this->service->saveAprova($params);
                   }
               }else if ($flag == 'R' || $flag == 'C') {
                    $params = array(
                        'P_IDSTATUS'            => 'PENDENTE',
                        'P_DSOBSERVACAO'        => $observacao,
                        'P_CDFILIAL'            => $row['FILIAL'],
                        'P_NRCONCPREORCAGOP'    => $row['NRCONCPREORCAGOP']
                    );
                    $this->service->updateConclusao($params);
                    
                    if($row['IDAPROVACAOGOP'] == 'APROVADO'){
                        $this->updateAprovacao($row, 'C', $observacao);
                    }else {
                        $this->updateAprovacao($row, 'R', $observacao);
                    }
               }
           }
           $this->service->commit();
           
           $retorno = '';
           $procedure = new StoredProcedure($this->connection, 'CONSISTE_ORCAMENTO_GOP');
    
           $procedure->addParam(new Param('P_CDFILDEST', Param::PARAM_INPUT, $rows[0]['CDFILIAL'],   Param::PARAM_TYPE_STR));
           $procedure->addParam(new Param('P_CDFILLANC', Param::PARAM_INPUT, null,                     Param::PARAM_TYPE_STR));
           $procedure->addParam(new Param('P_NRDEMANDA', Param::PARAM_INPUT, null,                     Param::PARAM_TYPE_STR));
           $procedure->addParam(new Param('P_RETORNO',   Param::PARAM_OUTPUT,$retorno,                 Param::PARAM_TYPE_STR));
           $procedure->addParam(new Param('P_NRORG',     Param::PARAM_INPUT, $this->environment->getNrorg(), Param::PARAM_TYPE_STR));
    
           $procedure->execute();
           
       }catch(\Exception $ex) {
           $this->service->rollback();
           $response->setError(new Error("Não foi possível processar os dados enviados" . $ex->getMessage(), 500));
       }
    }
    
    public function updateAprovacao($row, $flag, $observacao) {
        if($flag == 'A') {
            $row['IDAPROVACAOGOP'] = 'APROVADO';
        }else if($flag == 'C'){
            $row['IDAPROVACAOGOP'] = 'PENDENTE';
        }else if($flag == 'R') {
            $row['IDAPROVACAOGOP'] = 'REPROVADO';
        }
        
        $params = array(
            'P_NRAPROVAORCAGOP'     => $row['NRAPROVAORCAGOP'],
            'P_IDAPROVACAOGOP'      => $row['IDAPROVACAOGOP'],
            'P_DSOBSERVACAO'        => $observacao,
            'P_DTULTATU'            => $this->crudService->getEnvironment()->getCurrentTime()->format('d/m/Y H:i:s'),
            'P_NRORGULTATU'         => $this->environment->getNrorg(),
            'P_CDOPERULTATU'        => $this->environment->getCdOperador()
        );
       
        $this->service->updateAprova($params);
    }
    
    public function aprovaOrca(DTO\Request\Filter $request, DTO\Response $response) {
        $conditions = $request->getFilterCriteria()->getConditions();
        $rows = $this->crudService->getFilteredDataSet("gop11000_aprovaorcagop", $conditions)->getRows();
        $parents = array();

        foreach($rows as &$row) {
            if(!isset($parents[$row['NRORCAMENTOGOP']])) {
                $parents[$row['NRORCAMENTOGOP']] = array(
                    'ID'                => $row['NRORCAMENTOGOP'],
                    'NMFILIAL'          => $row['NMFILIAL'],
                    'CDFILIAL'          => $row['CDFILIAL'],
                    'ITENS'             => null,
                    'IDPERIODOORCA'     => $row['IDPERIODOORCA'],
                    'NRANOORCAGOP'      => $row['NRANOORCAGOP'],
                    'NRPERIODO'         => $row['NRPERIODO'],
                    'NRPERIODOORCA'     => $row['NRPERIODOORCA'],
                    'DSOBSERVACAO'      => $row['DSOBSERVACAO'],
                    'IDAPROVACAOGOP'    => $row['IDAPROVACAOGOP'],
                    'NRORCAMENTOGOP'    => $row['NRORCAMENTOGOP'],
                    'NRCONCPREORCAGOP'  => $row['NRCONCPREORCAGOP'],
                    'NRAPROVAORCAGOP'   => $row['NRAPROVAORCAGOP']
                );
            }
            $row['FILIAL']          = $row['CDFILIAL'];
            
            $row['NMFILIAL']        = null;
            $row['CDFILIAL']        = null;
            $row['IDPERIODOORCA']   = null;
            $row['NRANOORCAGOP']    = null;
            $row['NRPERIODO']       = null;
            $row['NRPERIODOORCA']   = null;
            
            $row['PARENT'] = $parents[$row['NRORCAMENTOGOP']]['ID'];
        };
        
        $datasetPais = array_values($parents);
        $datasetFilhos = array_values($rows);
        $dataset = array_merge($datasetPais, $datasetFilhos);
        
        $response->addDataSet(new DataSet("rows", $dataset));
    }

}

